/* test_tls.h
 *
 * Copyright (C) 2006-2026 wolfSSL Inc.
 *
 * This file is part of wolfSSL.
 *
 * wolfSSL is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * wolfSSL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335, USA
 */

#ifndef TESTS_API_TEST_TLS_H
#define TESTS_API_TEST_TLS_H

int test_utils_memio_move_message(void);
int test_tls12_unexpected_ccs(void);
int test_tls13_unexpected_ccs(void);
int test_tls12_curve_intersection(void);
int test_tls13_curve_intersection(void);
int test_tls_certreq_order(void);
int test_tls12_bad_cv_sig_alg(void);
int test_tls12_no_null_compression(void);
int test_tls_set_curves_list_ecc_fallback(void);

#define TEST_TLS_DECLS                                                         \
        TEST_DECL_GROUP("tls", test_utils_memio_move_message),                 \
        TEST_DECL_GROUP("tls", test_tls12_unexpected_ccs),                     \
        TEST_DECL_GROUP("tls", test_tls13_unexpected_ccs),                     \
        TEST_DECL_GROUP("tls", test_tls12_curve_intersection),                 \
        TEST_DECL_GROUP("tls", test_tls13_curve_intersection),                 \
        TEST_DECL_GROUP("tls", test_tls_certreq_order),                        \
        TEST_DECL_GROUP("tls", test_tls12_bad_cv_sig_alg),                     \
        TEST_DECL_GROUP("tls", test_tls12_no_null_compression),                \
        TEST_DECL_GROUP("tls", test_tls_set_curves_list_ecc_fallback)

#endif /* TESTS_API_TEST_TLS_H */
